@@ -9,6 +9,7 @@ recurring bugs we seem to encounter with high-level interaction.
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
 
+import asyncio
 import ast
 import os
 import signal
@@ -16,34 +17,25 @@ import shutil
 import sys
 import tempfile
 import unittest
-try:
-    from unittest import mock
-except ImportError:
-    import mock
+from unittest import mock
+
 from os.path import join
 
 import nose.tools as nt
 
 from IPython.core.error import InputRejected
 from IPython.core.inputtransformer import InputTransformer
+from IPython.core import interactiveshell
 from IPython.testing.decorators import (
     skipif, skip_win32, onlyif_unicode_paths, onlyif_cmds_exist,
 )
 from IPython.testing import tools as tt
 from IPython.utils.process import find_cmd
-from IPython.utils import py3compat
-from IPython.utils.py3compat import unicode_type, PY3
-
-if PY3:
-    from io import StringIO
-else:
-    from StringIO import StringIO
 
 #-----------------------------------------------------------------------------
 # Globals
 #-----------------------------------------------------------------------------
 # This is used by every single test, no point repeating it ad nauseam
-ip = get_ipython()
 
 #-----------------------------------------------------------------------------
 # Tests
@@ -134,8 +126,8 @@ class InteractiveShellTestCase(unittest.TestCase):
         """Pretty-printing lists of objects with non-ascii reprs may cause
         problems."""
         class Spam(object):
-          def __repr__(self):
-            return "\xe9"*50
+            def __repr__(self):
+                return "\xe9"*50
         import IPython.core.formatters
         f = IPython.core.formatters.PlainTextFormatter()
         f([Spam(),Spam()])
@@ -143,26 +135,14 @@ class InteractiveShellTestCase(unittest.TestCase):
 
     def test_future_flags(self):
         """Check that future flags are used for parsing code (gh-777)"""
-        ip.run_cell('from __future__ import print_function')
+        ip.run_cell('from __future__ import barry_as_FLUFL')
         try:
-            ip.run_cell('prfunc_return_val = print(1,2, sep=" ")')
+            ip.run_cell('prfunc_return_val = 1 <> 2')
             assert 'prfunc_return_val' in ip.user_ns
         finally:
             # Reset compiler flags so we don't mess up other tests.
             ip.compile.reset_compiler_flags()
 
-    def test_future_unicode(self):
-        """Check that unicode_literals is imported from __future__ (gh #786)"""
-        try:
-            ip.run_cell(u'byte_str = "a"')
-            assert isinstance(ip.user_ns['byte_str'], str) # string literals are byte strings by default
-            ip.run_cell('from __future__ import unicode_literals')
-            ip.run_cell(u'unicode_str = "a"')
-            assert isinstance(ip.user_ns['unicode_str'], unicode_type) # strings literals are now unicode
-        finally:
-            # Reset compiler flags so we don't mess up other tests.
-            ip.compile.reset_compiler_flags()
-    
     def test_can_pickle(self):
         "Can we pickle objects defined interactively (GH-29)"
         ip = get_ipython()
@@ -230,11 +210,13 @@ class InteractiveShellTestCase(unittest.TestCase):
         self.assertEqual(ip.var_expand(u'echo {f}'), u'echo Ca\xf1o')
         self.assertEqual(ip.var_expand(u'echo {f[:-1]}'), u'echo Ca\xf1')
         self.assertEqual(ip.var_expand(u'echo {1*2}'), u'echo 2')
+        
+        self.assertEqual(ip.var_expand(u"grep x | awk '{print $1}'"), u"grep x | awk '{print $1}'")
 
         ip.user_ns['f'] = b'Ca\xc3\xb1o'
         # This should not raise any exception:
         ip.var_expand(u'echo $f')
-    
+   
     def test_var_expand_local(self):
         """Test local variable expansion in !system and %magic calls"""
         # !system
@@ -280,6 +262,7 @@ class InteractiveShellTestCase(unittest.TestCase):
         pre_always = mock.Mock()
         post_explicit = mock.Mock()
         post_always = mock.Mock()
+        all_mocks = [pre_explicit, pre_always, post_explicit, post_always]
         
         ip.events.register('pre_run_cell', pre_explicit)
         ip.events.register('pre_execute', pre_always)
@@ -297,6 +280,19 @@ class InteractiveShellTestCase(unittest.TestCase):
             ip.run_cell("1")
             assert pre_explicit.called
             assert post_explicit.called
+            info, = pre_explicit.call_args[0]
+            result, = post_explicit.call_args[0]
+            self.assertEqual(info, result.info)
+            # check that post hooks are always called
+            [m.reset_mock() for m in all_mocks]
+            ip.run_cell("syntax error")
+            assert pre_always.called
+            assert pre_explicit.called
+            assert post_always.called
+            assert post_explicit.called
+            info, = pre_explicit.call_args[0]
+            result, = post_explicit.call_args[0]
+            self.assertEqual(info, result.info)
         finally:
             # remove post-exec
             ip.events.unregister('pre_run_cell', pre_explicit)
@@ -338,19 +334,6 @@ class InteractiveShellTestCase(unittest.TestCase):
         finally:
             trap.hook = save_hook
 
-    @skipif(sys.version_info[0] >= 3, "softspace removed in py3")
-    def test_print_softspace(self):
-        """Verify that softspace is handled correctly when executing multiple
-        statements.
-
-        In [1]: print 1; print 2
-        1
-        2
-
-        In [2]: print 1,; print 2
-        1 2
-        """
-        
     def test_ofind_line_magic(self):
         from IPython.core.magic import register_line_magic
         
@@ -466,22 +449,6 @@ class InteractiveShellTestCase(unittest.TestCase):
             # Reset the custom exception hook
             ip.set_custom_exc((), None)
     
-    @skipif(sys.version_info[0] >= 3, "no differences with __future__ in py3")
-    def test_future_environment(self):
-        "Can we run code with & without the shell's __future__ imports?"
-        ip.run_cell("from __future__ import division")
-        ip.run_cell("a = 1/2", shell_futures=True)
-        self.assertEqual(ip.user_ns['a'], 0.5)
-        ip.run_cell("b = 1/2", shell_futures=False)
-        self.assertEqual(ip.user_ns['b'], 0)
-        
-        ip.compile.reset_compiler_flags()
-        # This shouldn't leak to the shell's compiler
-        ip.run_cell("from __future__ import division \nc=1/2", shell_futures=False)
-        self.assertEqual(ip.user_ns['c'], 0.5)
-        ip.run_cell("d = 1/2", shell_futures=True)
-        self.assertEqual(ip.user_ns['d'], 0)
-
     def test_mktempfile(self):
         filename = ip.mktempfile()
         # Check that we can open the file again on Windows
@@ -509,15 +476,33 @@ class InteractiveShellTestCase(unittest.TestCase):
             raise DerivedInterrupt("foo")
         except KeyboardInterrupt:
             msg = ip.get_exception_only()
-        if sys.version_info[0] <= 2:
-            self.assertEqual(msg, 'DerivedInterrupt: foo\n')
-        else:
-            self.assertEqual(msg, 'IPython.core.tests.test_interactiveshell.DerivedInterrupt: foo\n')
+        self.assertEqual(msg, 'IPython.core.tests.test_interactiveshell.DerivedInterrupt: foo\n')
 
     def test_inspect_text(self):
         ip.run_cell('a = 5')
         text = ip.object_inspect_text('a')
-        self.assertIsInstance(text, unicode_type)
+        self.assertIsInstance(text, str)
+
+    def test_last_execution_result(self):
+        """ Check that last execution result gets set correctly (GH-10702) """
+        result = ip.run_cell('a = 5; a')
+        self.assertTrue(ip.last_execution_succeeded)
+        self.assertEqual(ip.last_execution_result.result, 5)
+
+        result = ip.run_cell('a = x_invalid_id_x')
+        self.assertFalse(ip.last_execution_succeeded)
+        self.assertFalse(ip.last_execution_result.success)
+        self.assertIsInstance(ip.last_execution_result.error_in_exec, NameError)
+
+    def test_reset_aliasing(self):
+        """ Check that standard posix aliases work after %reset. """
+        if os.name != 'posix':
+            return
+
+        ip.reset()
+        for cmd in ('clear', 'more', 'less', 'man'):
+            res = ip.run_cell('%' + cmd)
+            self.assertEqual(res.success, True)
 
 
 class TestSafeExecfileNonAsciiPath(unittest.TestCase):
@@ -529,7 +514,7 @@ class TestSafeExecfileNonAsciiPath(unittest.TestCase):
         os.mkdir(self.TESTDIR)
         with open(join(self.TESTDIR, u"åäötestscript.py"), "w") as sfile:
             sfile.write("pass\n")
-        self.oldpath = py3compat.getcwd()
+        self.oldpath = os.getcwd()
         os.chdir(self.TESTDIR)
         self.fname = u"åäötestscript.py"
 
@@ -544,6 +529,10 @@ class TestSafeExecfileNonAsciiPath(unittest.TestCase):
         ip.safe_execfile(self.fname, {}, raise_exceptions=True)
 
 class ExitCodeChecks(tt.TempFileMixin):
+
+    def setUp(self):
+        self.system = ip.system_raw
+
     def test_exit_code_ok(self):
         self.system('exit 0')
         self.assertEqual(ip.user_ns['_exit_code'], 0)
@@ -572,14 +561,18 @@ class ExitCodeChecks(tt.TempFileMixin):
             else:
                 del os.environ['SHELL']
 
-class TestSystemRaw(unittest.TestCase, ExitCodeChecks):
-    system = ip.system_raw
+
+class TestSystemRaw(ExitCodeChecks):
+
+    def setUp(self):
+        super().setUp()
+        self.system = ip.system_raw
 
     @onlyif_unicode_paths
     def test_1(self):
         """Test system_raw with non-ascii cmd
         """
-        cmd = u'''/usr/local/bin/python3.6 -c "'åäö'"   '''
+        cmd = u'''/usr/local/bin/python3.8 -c "'åäö'"   '''
         ip.system_raw(cmd)
 
     @mock.patch('subprocess.call', side_effect=KeyboardInterrupt)
@@ -593,8 +586,11 @@ class TestSystemRaw(unittest.TestCase, ExitCodeChecks):
         self.assertEqual(ip.user_ns['_exit_code'], -signal.SIGINT)
 
 # TODO: Exit codes are currently ignored on Windows.
-class TestSystemPipedExitCode(unittest.TestCase, ExitCodeChecks):
-    system = ip.system_piped
+class TestSystemPipedExitCode(ExitCodeChecks):
+
+    def setUp(self):
+        super().setUp()
+        self.system = ip.system_piped
 
     @skip_win32
     def test_exit_code_ok(self):
@@ -608,7 +604,7 @@ class TestSystemPipedExitCode(unittest.TestCase, ExitCodeChecks):
     def test_exit_code_signal(self):
         ExitCodeChecks.test_exit_code_signal(self)
 
-class TestModules(unittest.TestCase, tt.TempFileMixin):
+class TestModules(tt.TempFileMixin):
     def test_extraneous_loads(self):
         """Test we're not loading modules on startup that we shouldn't.
         """
@@ -622,10 +618,18 @@ class TestModules(unittest.TestCase, tt.TempFileMixin):
 
 class Negator(ast.NodeTransformer):
     """Negates all number literals in an AST."""
+
+    # for python 3.7 and earlier
     def visit_Num(self, node):
         node.n = -node.n
         return node
 
+    # for python 3.8+
+    def visit_Constant(self, node):
+        if isinstance(node.value, int):
+            return self.visit_Num(node)
+        return node
+
 class TestAstTransform(unittest.TestCase):
     def setUp(self):
         self.negator = Negator()
@@ -649,12 +653,12 @@ class TestAstTransform(unittest.TestCase):
             called.add(x)
         ip.push({'f':f})
         
-        with tt.AssertPrints("best of "):
+        with tt.AssertPrints("std. dev. of"):
             ip.run_line_magic("timeit", "-n1 f(1)")
         self.assertEqual(called, {-1})
         called.clear()
-        
-        with tt.AssertPrints("best of "):
+
+        with tt.AssertPrints("std. dev. of"):
             ip.run_cell_magic("timeit", "-n1 f(2)", "f(3)")
         self.assertEqual(called, {-2, -3})
     
@@ -685,14 +689,49 @@ class TestAstTransform(unittest.TestCase):
         with tt.AssertPrints("8"):
             ip.run_cell("amacro")
 
+class TestMiscTransform(unittest.TestCase):
+
+
+    def test_transform_only_once(self):
+        cleanup = 0
+        line_t = 0
+        def count_cleanup(lines):
+            nonlocal cleanup
+            cleanup += 1
+            return lines
+
+        def count_line_t(lines):
+            nonlocal line_t
+            line_t += 1
+            return lines
+
+        ip.input_transformer_manager.cleanup_transforms.append(count_cleanup)
+        ip.input_transformer_manager.line_transforms.append(count_line_t)
+
+        ip.run_cell('1')
+
+        assert cleanup == 1
+        assert line_t == 1
+
 class IntegerWrapper(ast.NodeTransformer):
     """Wraps all integers in a call to Integer()"""
+
+    # for Python 3.7 and earlier
+
+    # for Python 3.7 and earlier
     def visit_Num(self, node):
         if isinstance(node.n, int):
             return ast.Call(func=ast.Name(id='Integer', ctx=ast.Load()),
                             args=[node], keywords=[])
         return node
 
+    # For Python 3.8+
+    def visit_Constant(self, node):
+        if isinstance(node.value, int):
+            return self.visit_Num(node)
+        return node
+
+
 class TestAstTransform2(unittest.TestCase):
     def setUp(self):
         self.intwrapper = IntegerWrapper()
@@ -721,27 +760,36 @@ class TestAstTransform2(unittest.TestCase):
         def f(x):
             called.add(x)
         ip.push({'f':f})
-        
-        with tt.AssertPrints("best of "):
+
+        with tt.AssertPrints("std. dev. of"):
             ip.run_line_magic("timeit", "-n1 f(1)")
         self.assertEqual(called, {(1,)})
         called.clear()
-        
-        with tt.AssertPrints("best of "):
+
+        with tt.AssertPrints("std. dev. of"):
             ip.run_cell_magic("timeit", "-n1 f(2)", "f(3)")
         self.assertEqual(called, {(2,), (3,)})
 
 class ErrorTransformer(ast.NodeTransformer):
     """Throws an error when it sees a number."""
+
+    # for Python 3.7 and earlier
     def visit_Num(self, node):
         raise ValueError("test")
 
+    # for Python 3.8+
+    def visit_Constant(self, node):
+        if isinstance(node.value, int):
+            return self.visit_Num(node)
+        return node
+
+
 class TestAstTransformError(unittest.TestCase):
     def test_unregistering(self):
         err_transformer = ErrorTransformer()
         ip.ast_transformers.append(err_transformer)
         
-        with tt.AssertPrints("unregister", channel='stderr'):
+        with self.assertWarnsRegex(UserWarning, "It will be unregistered"):
             ip.run_cell("1 + 2")
         
         # This should have been removed.
@@ -754,10 +802,17 @@ class StringRejector(ast.NodeTransformer):
     Used to verify that NodeTransformers can signal that a piece of code should
     not be executed by throwing an InputRejected.
     """
-
+    
+    #for python 3.7 and earlier
     def visit_Str(self, node):
         raise InputRejected("test")
 
+    # 3.8 only
+    def visit_Constant(self, node):
+        if isinstance(node.value, str):
+            raise InputRejected("test")
+        return node
+
 
 class TestAstTransformInputRejection(unittest.TestCase):
 
@@ -850,36 +905,27 @@ def test_user_expression():
     
     # back to text only
     ip.display_formatter.active_types = ['text/plain']
-    
-
-
 
 
 class TestSyntaxErrorTransformer(unittest.TestCase):
     """Check that SyntaxError raised by an input transformer is handled by run_cell()"""
 
-    class SyntaxErrorTransformer(InputTransformer):
-
-        def push(self, line):
+    @staticmethod
+    def transformer(lines):
+        for line in lines:
             pos = line.find('syntaxerror')
             if pos >= 0:
                 e = SyntaxError('input contains "syntaxerror"')
                 e.text = line
                 e.offset = pos + 1
                 raise e
-            return line
-
-        def reset(self):
-            pass
+        return lines
 
     def setUp(self):
-        self.transformer = TestSyntaxErrorTransformer.SyntaxErrorTransformer()
-        ip.input_splitter.python_line_transforms.append(self.transformer)
-        ip.input_transformer_manager.python_line_transforms.append(self.transformer)
+        ip.input_transformers_post.append(self.transformer)
 
     def tearDown(self):
-        ip.input_splitter.python_line_transforms.remove(self.transformer)
-        ip.input_transformer_manager.python_line_transforms.remove(self.transformer)
+        ip.input_transformers_post.remove(self.transformer)
 
     def test_syntaxerror_input_transformer(self):
         with tt.AssertPrints('1234'):
@@ -892,25 +938,25 @@ class TestSyntaxErrorTransformer(unittest.TestCase):
             ip.run_cell('3456')
 
 
-
-def test_warning_suppression():
-    ip.run_cell("import warnings")
-    try:
-        with tt.AssertPrints("UserWarning: asdf", channel="stderr"):
-            ip.run_cell("warnings.warn('asdf')")
-        # Here's the real test -- if we run that again, we should get the
-        # warning again. Traditionally, each warning was only issued once per
-        # IPython session (approximately), even if the user typed in new and
-        # different code that should have also triggered the warning, leading
-        # to much confusion.
-        with tt.AssertPrints("UserWarning: asdf", channel="stderr"):
-            ip.run_cell("warnings.warn('asdf')")
-    finally:
-        ip.run_cell("del warnings")
+class TestWarningSuppression(unittest.TestCase):
+    def test_warning_suppression(self):
+        ip.run_cell("import warnings")
+        try:
+            with self.assertWarnsRegex(UserWarning, "asdf"):
+                ip.run_cell("warnings.warn('asdf')")
+            # Here's the real test -- if we run that again, we should get the
+            # warning again. Traditionally, each warning was only issued once per
+            # IPython session (approximately), even if the user typed in new and
+            # different code that should have also triggered the warning, leading
+            # to much confusion.
+            with self.assertWarnsRegex(UserWarning, "asdf"):
+                ip.run_cell("warnings.warn('asdf')")
+        finally:
+            ip.run_cell("del warnings")
 
 
-def test_deprecation_warning():
-    ip.run_cell("""
+    def test_deprecation_warning(self):
+        ip.run_cell("""
 import warnings
 def wrn():
     warnings.warn(
@@ -918,17 +964,17 @@ def wrn():
         DeprecationWarning
     )
         """)
-    try:
-        with tt.AssertPrints("I AM  A WARNING", channel="stderr"):
-            ip.run_cell("wrn()")
-    finally:
-        ip.run_cell("del warnings")
-        ip.run_cell("del wrn")
+        try:
+            with self.assertWarnsRegex(DeprecationWarning, "I AM  A WARNING"):
+                ip.run_cell("wrn()")
+        finally:
+            ip.run_cell("del warnings")
+            ip.run_cell("del wrn")
 
 
 class TestImportNoDeprecate(tt.TempFileMixin):
 
-    def setup(self):
+    def setUp(self):
         """Make a valid python temp file."""
         self.mktmp("""
 import warnings
@@ -938,6 +984,7 @@ def wrn():
         DeprecationWarning
     )
 """)
+        super().setUp()
 
     def test_no_dep(self):
         """
@@ -959,3 +1006,37 @@ def test_custom_exc_count():
     ip.set_custom_exc((), None)
     nt.assert_equal(hook.call_count, 1)
     nt.assert_equal(ip.execution_count, before + 1)
+
+
+def test_run_cell_async():
+    loop = asyncio.get_event_loop()
+    ip.run_cell("import asyncio")
+    coro = ip.run_cell_async("await asyncio.sleep(0.01)\n5")
+    assert asyncio.iscoroutine(coro)
+    result = loop.run_until_complete(coro)
+    assert isinstance(result, interactiveshell.ExecutionResult)
+    assert result.result == 5
+
+
+def test_should_run_async():
+    assert not ip.should_run_async("a = 5")
+    assert ip.should_run_async("await x")
+    assert ip.should_run_async("import asyncio; await asyncio.sleep(1)")
+
+
+def test_set_custom_completer():
+    num_completers = len(ip.Completer.matchers)
+
+    def foo(*args, **kwargs):
+        return "I'm a completer!"
+
+    ip.set_custom_completer(foo, 0)
+
+    # check that we've really added a new completer
+    assert len(ip.Completer.matchers) == num_completers + 1
+
+    # check that the first completer is the function we defined
+    assert ip.Completer.matchers[0]() == "I'm a completer!"
+
+    # clean up
+    ip.Completer.custom_matchers.pop()
